package mhtree;

import messif.objects.LocalAbstractObject;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

public class Histogram {
    private final HashMap<Integer, HashMap<Long, Integer>> levelToHullsCoveredToObjectCount = new HashMap<>();

    Histogram() {
    }

    public static Histogram generate(Node x) {
        Histogram histogram = new Histogram();

        for (int level = 1; level <= x.getHeight() + 1; level++) {
            Set<Node> levelNodes = x.getNodesOnLevel(level);

            for (LocalAbstractObject object : x.getObjects()) {
                long coveredObjectCount = levelNodes.stream()
                        .filter(node -> node.isCovered(object))
                        .count();

                histogram.addCoveredCount(level, coveredObjectCount);
            }
        }

        return histogram;
    }

    public void addCoveredCount(int level, long count) {
        levelToHullsCoveredToObjectCount.merge(
                level,
                new HashMap<>(Collections.singletonMap(count, 1)),
                (hullsCoveredToObjectCount, x) -> {
                    hullsCoveredToObjectCount.merge(count, 1, (objectCount, y) -> objectCount + 1);
                    return hullsCoveredToObjectCount;
                });
    }

    @Override
    public String toString() {
        StringBuilder stringBuilder = new StringBuilder();

        levelToHullsCoveredToObjectCount.forEach((level, hullsCoveredToObjectCount) -> {
            stringBuilder.append("Level ");
            stringBuilder.append(level);
            stringBuilder.append('\n');

            hullsCoveredToObjectCount.forEach((hullsCovered, objectCount) -> {
                stringBuilder.append(objectCount);
                stringBuilder.append(" object");
                if (objectCount != 1) stringBuilder.append('s');

                if (hullsCovered == 0) {
                    stringBuilder.append(" not covered");
                } else {
                    stringBuilder.append(" is covered by ");
                    stringBuilder.append(hullsCovered);
                    stringBuilder.append(" hull");
                    if (hullsCovered != 1) stringBuilder.append('s');
                }

                stringBuilder.append('\n');
            });
        });

        return stringBuilder.toString();
    }
}
