package mhtree;

import cz.muni.fi.disa.similarityoperators.cover.AbstractRepresentation.PrecomputedDistances;
import messif.objects.LocalAbstractObject;

import java.io.Serializable;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

public class InternalNode extends Node implements Serializable {

    /**
     * Serialization ID
     */
    private static final long serialVersionUID = 2L;

    private final Set<Node> children;

    InternalNode(PrecomputedDistances distances, InsertType insertType, ObjectToNodeDistance objectToNodeDistance) {
        super(distances, insertType, objectToNodeDistance);
        children = new HashSet<>();
    }

    public void addChildren(Set<Node> children) {
        this.children.addAll(children);
    }

    public Set<Node> getChildren() {
        return children;
    }

    public Node getNearestChild(LocalAbstractObject object) {
        Map<Node, Double> nodeToObjectDistance = children.stream()
                .collect(Collectors.toMap(Function.identity(), node -> node.getDistance(object)));

        return Collections.min(nodeToObjectDistance.entrySet(), Map.Entry.comparingByValue()).getKey();
    }

    public void addObject(LocalAbstractObject object) {
        addNewObject(object);
    }

    public Set<LocalAbstractObject> getObjects() {
        return children.stream()
                .map(Node::getObjects)
                .flatMap(Collection::stream)
                .collect(Collectors.toSet());
    }

    public boolean contains(LocalAbstractObject object) {
        return children.stream().anyMatch(child -> child.contains(object));
    }

    public int getHeight() {
        return children.stream()
                .mapToInt(Node::getHeight)
                .summaryStatistics()
                .getMax() + 1;
    }

    public Set<Node> getNodesOnLevel(int level) {
        if (getLevel() == level) return Collections.singleton(this);

        return children.stream()
                .map(child -> child.getNodesOnLevel(level))
                .flatMap(Collection::stream)
                .collect(Collectors.toSet());
    }
}
